# -*- coding: UTF-8 -*-
from setuphelpers import *
import requests

r"""

Installation procedure: https://firefox-source-docs.mozilla.org/browser/installer/windows/installer/FullConfig.html

"""
# Declaring global variables - Warnings: 1) WAPT context is only available in package functions; 2) Global variables are not persistent between calls
bin_name_string = "Thunderbird_Setup_%s.exe"
silent_args = "/S /PreventRebootRequired=false /TaskbarShortcut=false /DesktopShortcut=false /StartMenuShortcut=true /MaintenanceService=false /RegisterDefaultAgent=false /RemoveDistributionDir=false /OptionalExtensions=false"
list_lang = ["it", "fr", "en-US", "es-ES", "de"]
policies_path = makepath(programfiles, "Mozilla Thunderbird", "distribution", "policies.json")


def install():
    # Initializing variables
    package_version = control.version.split("-")[0]
    bin_name = bin_name_string % package_version
    app_arch = control.architecture
    app_processes_list = control.impacted_process.split(",")
    # Translating arch
    if app_arch == "x64":
        arch_to_uninstall = "x86"
    else:
        arch_to_uninstall = "x64"
    # Translating locale
    for lang in list_lang:
        if control.locale in lang:
            locale = lang
    app_uninstallkey = "Mozilla Thunderbird %s (%s %s)" % (package_version, app_arch, locale)

    # Uninstalling others versions
    for uninstall in installed_softwares(name="Mozilla Thunderbird"):
        if arch_to_uninstall in uninstall["name"]:
            print("Removing: %s" % (uninstall["name"]))
            killalltasks(app_processes_list)
            run(uninstall_cmd(uninstall["key"]))
            wait_uninstallkey_absent(uninstall["key"])

    # Installing the package
    print("Installing: %s" % bin_name)
    install_exe_if_needed(bin_name, silentflags=silent_args, key=app_uninstallkey, min_version=package_version)

    # Making sure that Mozilla Maintenance Service is not installed
    for uninstall in installed_softwares(name="Mozilla Maintenance Service"):
        print("Removing: %s" % (uninstall["name"]))
        run(uninstall_cmd(uninstall["key"]))

    # Adding Minimal configuration for Thunderbird you may use the package "tis-config-thunderbird" for further options
    if not isfile(policies_path):
        registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Mozilla\Thunderbird", "DisableAppUpdate", 1, REG_DWORD)
        registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Mozilla\Thunderbird", "DisableTelemetry", 1, REG_DWORD)


def update_package():
    # Declaring local variables
    result = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    app_name = control.name
    arch = {"x64": "win64", "x86": "win", "all": "win"}

    # Downloading
    lang = control.locale
    # Translating locale
    if "en" in lang:
        lang = "en-US"
    if "es" in lang:
        lang = "es-ES"

    url_dl = requests.head(
        "https://download.mozilla.org/?product=thunderbird-latest&os=%s&lang=%s" % (arch[control.architecture], lang), proxies=proxies
    ).headers["Location"]
    latest_bin = url_dl.rsplit("/", 1)[1].replace("%20", "_")
    version = url_dl.split("/")[-4]

    print("Latest %s version is: %s" % (app_name, version))
    print("Download URL is: %s" % url_dl)

    # Downloading latest binaries
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(url_dl, latest_bin, proxies=proxies)

    # Changing version of the package
    if Version(version) > Version(control.get_software_version()):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        result = True
    control.version = "%s-%s" % (Version(version), control.version.split("-", 1)[-1])
    # control.set_software_version(version)
    control.save_control_to_wapt()

    # Deleting outdated binaries
    remove_outdated_binaries(version)

    # Validating update-package-sources
    return result
